/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    :   adslAtucChanTable_access.c
 * Author       :   Subramani
 * Date         :   17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      :   Amazon
 * Block        :   ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                  1) Getting the data context pointer & loop context
 *                  pointer
 *                  2) User-defined data access functions (per column) for data
 *                  in table adslAtucChanTable
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */



/* ===========================================================================
 * Revision History:
 *   Subramani: Data type for long_ret is changed from long to u_int32
 *
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.access_functions.conf,v 1.6 2003/09/11 22:48:49 rstory Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>

#include "adslAtucChanTable_access.h"

#ifdef STUB_IOCTL
#include "adslAtucChanTable_enums.h"
#endif

/* column number definitions for table adslAtucChanTable */
#include "adslAtucChanTable_columns.h"

/*
 * ===========================================================================
 *                          GLOBAL DEFINITIONS
 * ===========================================================================
*/

// *** Global variable definitions
//static long long_ret;
static u_int32 long_ret;

/*
 * ===========================================================================
 *                          LOCAL FUNCTIONS
 * ===========================================================================
*/

/** returns the first data point within the adslAtucChanTable table data.

    Set the my_loop_context variable to the first data point structure
    of your choice (from which you can find the next one).  This could
    be anything from the first node in a linked list, to an integer
    pointer containing the beginning of an array variable.

    Set the my_data_context variable to something to be returned to
    you later that will provide you with the data to return in a given
    row.  This could be the same pointer as what my_loop_context is
    set to, or something different.

    The put_index_data variable contains a list of snmp variable
    bindings, one for each index in your table.  Set the values of
    each appropriately according to the data matching the first row
    and return the put_index_data variable at the end of the function.
*/
netsnmp_variable_list *
adslAtucChanTable_get_first_data_point(void **my_loop_context,
                                       void **my_data_context,
                                       netsnmp_variable_list *
                                       put_index_data,
                                       netsnmp_iterator_info *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-First: adslAtucChanTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }

    *My_Data_Context=IFTYPE_ADSL_FAST_INTERLEAVE;
    *my_data_context=My_Data_Context;

    put_index_data = adslTable_get_first_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/** functionally the same as adslAtucChanTable_get_first_data_point, but
   my_loop_context has already been set to a previous value and should
   be updated to the next in the list.  For example, if it was a
   linked list, you might want to cast it to your local data type and
   then return my_loop_context->next.  The my_data_context pointer
   should be set to something you need later and the indexes in
   put_index_data updated again. */
netsnmp_variable_list *
adslAtucChanTable_get_next_data_point(void **my_loop_context,
                                      void **my_data_context,
                                      netsnmp_variable_list *
                                      put_index_data,
                                      netsnmp_iterator_info *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-Next: adslAtucChanTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }
    *My_Data_Context=IFTYPE_ADSL_FAST_INTERLEAVE;
    *my_data_context=My_Data_Context;

     put_index_data = adslTable_get_next_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}


/*
 * User-defined data access functions (per column) for data in table
 * adslAtucChanTable
 */

/*
 * NOTE:
 * - these get_ routines MUST return data that will not be freed (ie,
 *   use static variables or persistent data).  It will be copied, if
 *   needed, immediately after the get_ routine has been called.
 * - these SET routines must copy the incoming data and can not take
 *   ownership of the memory passed in by the val pointer.
 */
#if defined(HAVE_OPTIMIZED_CODE)

u_int32         *
get_adslAtucChanInfo(void *data_context, size_t * ret_len,
		unsigned int var)
{
  unsigned int flags = 0;	
  struct adslAtucChanInfo adslAtucChanStruct;
  memset(&adslAtucChanStruct,0,sizeof(adslAtucChanInfo));
  adslAtucChanStruct.ifIndex= (*(int *)data_context);

  switch (var)  {

        case COLUMN_ADSLATUCCHANINTERLEAVEDELAY:
	   flags = ATUC_CHAN_INTLV_DELAY_FLAG;
	   break;

        case COLUMN_ADSLATUCCHANCURRTXRATE:
	   flags = ATUC_CHAN_CURR_TX_RATE_FLAG;
	   break;

        case COLUMN_ADSLATUCCHANPREVTXRATE:
	   flags = ATUC_CHAN_PREV_TX_RATE_FLAG;
	   break;

     	default:
     	   DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
     	   return NULL;
  }

  SET_FLAG(&adslAtucChanStruct.flags, flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INFO, &adslAtucChanStruct);
#else
    get_adslAtucChanTable(Fd, GET_ADSL_ATUC_CHAN_INFO,
        (void *) &adslAtucChanStruct);
#endif
    if (IS_FLAG_SET(&adslAtucChanStruct.flags, flags) == 0)
       DEBUGMSGTL(("adslMIB-ATUC", "ChanTable: GET- %x CMV fail\n", flags));

    switch (flags)  {
	    
	case ATUC_CHAN_INTLV_DELAY_FLAG:
    	   long_ret = adslAtucChanStruct.interleaveDelay;
	   break;

	case ATUC_CHAN_CURR_TX_RATE_FLAG:
     	   long_ret = adslAtucChanStruct.currTxRate;
	   break;

	case ATUC_CHAN_PREV_TX_RATE_FLAG:
    	   long_ret = adslAtucChanStruct.prevTxRate;
	   break;

        default:
     	    DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
            return NULL;
    }
    
    CLR_FLAG(&adslAtucChanStruct.flags, flags);
    *ret_len = sizeof(u_int32);
    return (u_int32 *) &long_ret;
}

#else

/** XXX: return a data pointer to the data for the adslAtucChanInterleaveDelay
    column and set ret_len to its proper size in bytes. */

u_int32         *
get_adslAtucChanInterleaveDelay(void *data_context, size_t * ret_len)
{
    struct adslAtucChanInfo adslAtucChanStruct;

    memset(&adslAtucChanStruct,0,sizeof(adslAtucChanInfo));
    adslAtucChanStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAtucChanStruct.flags, ATUC_CHAN_INTLV_DELAY_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INFO, &adslAtucChanStruct);
#else
    get_adslAtucChanTable(Fd, GET_ADSL_ATUC_CHAN_INFO,
        (void *) &adslAtucChanStruct);
#endif
    if (IS_FLAG_SET(&adslAtucChanStruct.flags, ATUC_CHAN_INTLV_DELAY_FLAG) == 0)
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntlvDelay CMV fail\n"));

    CLR_FLAG(&adslAtucChanStruct.flags, ATUC_CHAN_INTLV_DELAY_FLAG);
    long_ret=adslAtucChanStruct.interleaveDelay;
    *ret_len = sizeof(adslAtucChanStruct.interleaveDelay);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucChanCurrTxRate column
    and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAtucChanCurrTxRate(void *data_context, size_t * ret_len)
{
    struct adslAtucChanInfo adslAtucChanStruct;

    memset(&adslAtucChanStruct,0,sizeof(adslAtucChanInfo));
    adslAtucChanStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAtucChanStruct.flags, ATUC_CHAN_CURR_TX_RATE_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INFO, &adslAtucChanStruct);
#else
    get_adslAtucChanTable(Fd, GET_ADSL_ATUC_CHAN_INFO,
        (void *) &adslAtucChanStruct);
#endif
   if (IS_FLAG_SET(&adslAtucChanStruct.flags, ATUC_CHAN_CURR_TX_RATE_FLAG) == 0)
       DEBUGMSGTL(("adslMIB-ATUC", "ChanCurrTxRate CMV fail\n"));

    CLR_FLAG(&adslAtucChanStruct.flags, ATUC_CHAN_CURR_TX_RATE_FLAG);
     long_ret=adslAtucChanStruct.currTxRate;
    *ret_len = sizeof(adslAtucChanStruct.currTxRate);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucChanPrevTxRate column
    and set ret_len to its proper size in bytes. */

u_int32         *
get_adslAtucChanPrevTxRate(void *data_context, size_t * ret_len)
{
    struct adslAtucChanInfo adslAtucChanStruct;

    memset(&adslAtucChanStruct,0,sizeof(adslAtucChanInfo));
    adslAtucChanStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAtucChanStruct.flags, ATUC_CHAN_PREV_TX_RATE_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_CHAN_INFO, &adslAtucChanStruct);
#else
    get_adslAtucChanTable(Fd, GET_ADSL_ATUC_CHAN_INFO,
        (void *) &adslAtucChanStruct);
#endif
   if (IS_FLAG_SET(&adslAtucChanStruct.flags, ATUC_CHAN_PREV_TX_RATE_FLAG) == 0)
       DEBUGMSGTL(("adslMIB-ATUC", "ChanPrevTxRate CMV fail\n"));

    CLR_FLAG(&adslAtucChanStruct.flags, ATUC_CHAN_PREV_TX_RATE_FLAG);
    long_ret=adslAtucChanStruct.prevTxRate;
    *ret_len = sizeof(adslAtucChanStruct.prevTxRate);
    return (u_int32 *) &long_ret;
}

#endif


#ifndef ENABLE_DRIVER_CALL
int  get_adslAtucChanTable(int Fd2,int GET_ADSL_atuc_CHAN_INFO,
        void *adslAtucChanTable)
{
    struct adslAtucChanInfo *adslAtucChanValue;
    adslAtucChanValue=(adslAtucChanInfo *)adslAtucChanTable;
    adslAtucChanValue->interleaveDelay=30;
    adslAtucChanValue->currTxRate=30;
    adslAtucChanValue->prevTxRate=30;
    return 0;
}
#endif
