/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    :   adslAtucIntervalTable_access.c
 * Author       :   Subramani
 * Date         :   17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      :   Amazon
 * Block        :   ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                  1) Getting the data context pointer & loop context
 *                  pointer
 *                  2) User-defined data access functions (per column) for data
 *                  in table adslAtucIntervalTable
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */



/* ===========================================================================
 * Revision History:
 * Subramani: Return type for get_adslAtucIntervalValidData changed from long
 *   to int
 *
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.access_functions.conf,v 1.6 2003/09/11 22:48:49 rstory Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslAtucIntervalTable_access.h"
#include "adslAtucIntervalTable_enums.h"

/** column number definitions for table adslAtucIntervalTable **/
#include "adslAtucIntervalTable_columns.h"

/*
 * ===========================================================================
 *                          GLOBAL DEFINITIONS
 * ===========================================================================
*/

// *** Global variable definitions
static u_int32 long_ret;

/*
 * ===========================================================================
 *                          LOCAL FUNCTIONS
 * ===========================================================================
*/

/** returns the first data point within the adslAtucIntervalTable table data.

    Set the my_loop_context variable to the first data point structure
    of your choice (from which you can find the next one).  This could
    be anything from the first node in a linked list, to an integer
    pointer containing the beginning of an array variable.

    Set the my_data_context variable to something to be returned to
    you later that will provide you with the data to return in a given
    row.  This could be the same pointer as what my_loop_context is
    set to, or something different.

    The put_index_data variable contains a list of snmp variable
    bindings, one for each index in your table.  Set the values of
    each appropriately according to the data matching the first row
    and return the put_index_data variable at the end of the function.
*/
netsnmp_variable_list *
adslAtucIntervalTable_get_first_data_point(void **my_loop_context,
                                           void **my_data_context,
                                           netsnmp_variable_list *
                                           put_index_data,
                                           netsnmp_iterator_info *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-First: adslAtucIntervalTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }

    *My_Data_Context=IFTYPE_ADSL;
    *my_data_context=My_Data_Context;

    put_index_data = adslTable_get_first_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/** functionally the same as adslAtucIntervalTable_get_first_data_point, but
   my_loop_context has already been set to a previous value and should
   be updated to the next in the list.  For example, if it was a
   linked list, you might want to cast it to your local data type and
   then return my_loop_context->next.  The my_data_context pointer
   should be set to something you need later and the indexes in
   put_index_data updated again. */
netsnmp_variable_list *
adslAtucIntervalTable_get_next_data_point(void **my_loop_context,
                                          void **my_data_context,
                                          netsnmp_variable_list *
                                          put_index_data,
                                          netsnmp_iterator_info *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-Next: adslAtucIntervalTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }
    *My_Data_Context=IFTYPE_ADSL;
    *my_data_context=My_Data_Context;

     put_index_data = adslTable_get_next_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/*
 * User-defined data access functions (per column) for data in table
 * adslAtucIntervalTable
 */
/*
 * NOTE:
 * - these get_ routines MUST return data that will not be freed (ie,
 *   use static variables or persistent data).  It will be copied, if
 *   needed, immediately after the get_ routine has been called.
 * - these SET routines must copy the incoming data and can not take
 *   ownership of the memory passed in by the val pointer.
 */

#if defined(HAVE_OPTIMIZED_CODE)

u_int32      *
get_adslAtucIntervalData(void *data_context, size_t * ret_len, 
		unsigned int var)
{
   unsigned int flags = 0;
   struct adslAtucIntvlInfo adslAtucIntvlStruct;
   memset(&adslAtucIntvlStruct,0,sizeof(adslAtucIntvlInfo));
   adslAtucIntvlStruct.ifIndex= (*(int *)data_context);
   adslAtucIntvlStruct.IntervalNumber= (*(int *)ret_len);
  
  switch (var) {

    case COLUMN_ADSLATUCINTERVALLOFS:
  	flags = ATUC_INTVL_LOF_FLAG;
        break;
	
    case COLUMN_ADSLATUCINTERVALLOSS:
	flags = ATUC_INTVL_LOS_FLAG;
	break;
	
    case COLUMN_ADSLATUCINTERVALESS:
	flags = ATUC_INTVL_ESS_FLAG;
	break;

    case COLUMN_ADSLATUCINTERVALINITS:
	flags = ATUC_INTVL_INIT_FLAG;
	break;
	
    default:
     	DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
        return NULL;
  }

  SET_FLAG(&adslAtucIntvlStruct.flags, flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_INTVL_INFO, &adslAtucIntvlStruct);
#else
    get_adslAtucIntervalTable(Fd, GET_ADSL_ATUC_INTVL_INFO,
        (void *) &adslAtucIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAtucIntvlStruct.flags, flags) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "IntervalTable: GET- %x CMV fail\n", flags));

    switch (flags)  {

	 case ATUC_INTVL_LOF_FLAG:
    	    long_ret = adslAtucIntvlStruct.intervalLOF;
            break;
	 
	 case ATUC_INTVL_LOS_FLAG:
    	    long_ret = adslAtucIntvlStruct.intervalLOS;
	    break;
	  
	 case ATUC_INTVL_ESS_FLAG:
    	    long_ret = adslAtucIntvlStruct.intervalES;
	    break;
	  
	 case ATUC_INTVL_INIT_FLAG:
    	    long_ret = adslAtucIntvlStruct.intervalInits;
	    break;

         default:
     	    DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
            return NULL;
     }
     
    CLR_FLAG(&adslAtucIntvlStruct.flags, flags);
    *ret_len = sizeof(u_int32);
    return (u_int32 *) &long_ret;
}

#else
/** XXX: return a data pointer to the data for the adslAtucIntervalLofs
    column and set ret_len to its proper size in bytes. */

u_int32      *
get_adslAtucIntervalLofs(void *data_context, size_t * ret_len)
{
    struct adslAtucIntvlInfo adslAtucIntvlStruct;

    memset(&adslAtucIntvlStruct,0,sizeof(adslAtucIntvlInfo));
    adslAtucIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_LOF_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_INTVL_INFO, &adslAtucIntvlStruct);
#else
    get_adslAtucIntervalTable(Fd, GET_ADSL_ATUC_INTVL_INFO,
        (void *) &adslAtucIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAtucIntvlStruct.flags, ATUC_INTVL_LOF_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlLOF CMV fail\n"));

    CLR_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_LOF_FLAG);
    long_ret = adslAtucIntvlStruct.intervalLOF;
    *ret_len = sizeof(adslAtucIntvlStruct.intervalLOF);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucIntervalLoss column
    and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAtucIntervalLoss(void *data_context, size_t * ret_len)
{
        struct adslAtucIntvlInfo adslAtucIntvlStruct;

        memset(&adslAtucIntvlStruct,0,sizeof(adslAtucIntvlInfo));
    adslAtucIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_LOS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_INTVL_INFO, &adslAtucIntvlStruct);
#else
    get_adslAtucIntervalTable(Fd, GET_ADSL_ATUC_INTVL_INFO,
        (void *) &adslAtucIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAtucIntvlStruct.flags, ATUC_INTVL_LOS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlLOS CMV fail\n"));

    CLR_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_LOS_FLAG);
    long_ret = adslAtucIntvlStruct.intervalLOS;
    *ret_len = sizeof(adslAtucIntvlStruct.intervalLOS);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucIntervalESs column
    and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAtucIntervalESs(void *data_context, size_t * ret_len)
{
    struct adslAtucIntvlInfo adslAtucIntvlStruct;

    memset(&adslAtucIntvlStruct,0,sizeof(adslAtucIntvlInfo));
    adslAtucIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_ESS_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_INTVL_INFO, &adslAtucIntvlStruct);
#else
    get_adslAtucIntervalTable(Fd, GET_ADSL_ATUC_INTVL_INFO,
        (void *) &adslAtucIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAtucIntvlStruct.flags, ATUC_INTVL_ESS_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlESS CMV fail\n"));

    CLR_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_ESS_FLAG);
    long_ret = adslAtucIntvlStruct.intervalES;
    *ret_len = sizeof(adslAtucIntvlStruct.intervalES);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAtucIntervalInits column
    and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAtucIntervalInits(void *data_context, size_t * ret_len)
{
    struct adslAtucIntvlInfo adslAtucIntvlStruct;

    memset(&adslAtucIntvlStruct,0,sizeof(adslAtucIntvlInfo));
    adslAtucIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_INIT_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_INTVL_INFO, &adslAtucIntvlStruct);
#else
    get_adslAtucIntervalTable(Fd, GET_ADSL_ATUC_INTVL_INFO,
        (void *) &adslAtucIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAtucIntvlStruct.flags, ATUC_INTVL_INIT_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlInit CMV fail\n"));
       
    CLR_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_INIT_FLAG);
    long_ret = adslAtucIntvlStruct.intervalInits;
    *ret_len = sizeof(adslAtucIntvlStruct.intervalInits);
    return (u_int32 *) &long_ret;
}

#endif

/** XXX: return a data pointer to the data for the adslAtucIntervalValidData
    column and set ret_len to its proper size in bytes. */
int           *
get_adslAtucIntervalValidData(void *data_context, size_t * ret_len)
{
    struct adslAtucIntvlInfo adslAtucIntvlStruct;

    memset(&adslAtucIntvlStruct,0,sizeof(adslAtucIntvlInfo));
    adslAtucIntvlStruct.ifIndex= (*(int *)data_context);
    adslAtucIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_VALID_DATA_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUC_INTVL_INFO, &adslAtucIntvlStruct);
#else
    get_adslAtucIntervalTable(Fd, GET_ADSL_ATUC_INTVL_INFO,
        (void *) &adslAtucIntvlStruct);
#endif
   if (IS_FLAG_SET(&adslAtucIntvlStruct.flags, ATUC_INTVL_VALID_DATA_FLAG) == 0)
       DEBUGMSGTL(("adslMIB-ATUC", "ChanIntvlValidData CMV fail\n"));

    CLR_FLAG(&adslAtucIntvlStruct.flags, ATUC_INTVL_VALID_DATA_FLAG);
    long_ret = adslAtucIntvlStruct.intervalValidData;
    *ret_len = sizeof(adslAtucIntvlStruct.intervalValidData);
    return (int *) &long_ret;
}

#ifndef ENABLE_DRIVER_CALL
int get_adslAtucIntervalTable(int Fd2,int GET_adsl_ATUC_INTVL_INFO2,
        void *adslAtucInvtlTable)
{
        struct adslAtucIntvlInfo *adslAtucInvtlValue;
        adslAtucInvtlValue = (adslAtucIntvlInfo *)adslAtucInvtlTable;
        adslAtucInvtlValue->intervalLOF = 10;
        adslAtucInvtlValue->intervalLOS = 20;
        adslAtucInvtlValue->intervalES = 30;
        adslAtucInvtlValue->intervalInits = 40;
        adslAtucInvtlValue->intervalValidData = 1;
        return 0;
}

#endif

