/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    : adslAturIntervalTable_access.c
 * Author       : Subbi
 * Date         : 17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      : Amazon
 * Block        : ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                 1) Getting the data context pointer & loop context pointer
 *                 2) User-defined data access functions (per column) for data
 *                    in table adslAturIntervalTable
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */

/* ===========================================================================
 * Revision History:
 * Subramani: Changed the return types of get_adslAturIntervalValidData
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.access_functions.conf,v 1.6 2003/09/11 22:48:49 rstory Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
 */
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslAturIntervalTable_access.h"
#include "adslAturIntervalTable_enums.h"

/** column number definitions for table adslAturIntervalTable **/
#include "adslAturIntervalTable_columns.h"

/*
 * ===========================================================================
 *                           GLOBAL DEFINITIONS
 * ===========================================================================
 */
// *** Global variable definitions
static long long_ret;


/*
 * ===========================================================================
 *                           LOCAL FUNCTIONS
 * ===========================================================================
 */
/** returns the first data point within the adslAturIntervalTable table data.

    Set the my_loop_context variable to the first data point structure
    of your choice (from which you can find the next one).  This could
    be anything from the first node in a linked list, to an integer
    pointer containing the beginning of an array variable.

    Set the my_data_context variable to something to be returned to
    you later that will provide you with the data to return in a given
    row.  This could be the same pointer as what my_loop_context is
    set to, or something different.

    The put_index_data variable contains a list of snmp variable
    bindings, one for each index in your table.  Set the values of
    each appropriately according to the data matching the first row
    and return the put_index_data variable at the end of the function.
*/
netsnmp_variable_list *
adslAturIntervalTable_get_first_data_point(void **my_loop_context,
                                           void **my_data_context,
                                           netsnmp_variable_list *
                                           put_index_data,
                                           netsnmp_iterator_info *mydata)
{

   int *My_Data_Context;
   My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
   if (!My_Data_Context) {
       put_index_data = NULL;
       snmp_log(LOG_ERR, "adslAturIntervalTable: My_Data_Context malloc \
                    failed\n");
       return put_index_data;
   }
   *My_Data_Context=IFTYPE_ADSL;
   *my_data_context=My_Data_Context;

   put_index_data = adslTable_get_first_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/** functionally the same as adslAturIntervalTable_get_first_data_point, but
   my_loop_context has already been set to a previous value and should
   be updated to the next in the list.  For example, if it was a
   linked list, you might want to cast it to your local data type and
   then return my_loop_context->next.  The my_data_context pointer
   should be set to something you need later and the indexes in
   put_index_data updated again. */
netsnmp_variable_list *
adslAturIntervalTable_get_next_data_point(void **my_loop_context,
                                          void **my_data_context,
                                          netsnmp_variable_list *
                                          put_index_data,
                                          netsnmp_iterator_info *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
       put_index_data = NULL;
       snmp_log(LOG_ERR, "adslAturIntervalTable: My_Data_Context malloc \
                    failed\n");
       return put_index_data;
   }
   *My_Data_Context=IFTYPE_ADSL;
   *my_data_context=My_Data_Context;

   put_index_data = adslTable_get_next_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

   SNMP_FREE(My_Data_Context);
   return put_index_data;
}


/*  User-defined data access functions (per column) for data in table
    adslAturIntervalTable */
/*
 * NOTE:
 * - these get_ routines MUST return data that will not be freed (ie,
 *   use static variables or persistent data).  It will be copied, if
 *   needed, immediately after the get_ routine has been called.
 * - these SET routines must copy the incoming data and can not take
 *   ownership of the memory passed in by the val pointer.
 */

#if defined(HAVE_OPTIMIZED_CODE)

u_int32      *
get_adslAturIntervalData(void *data_context, size_t * ret_len, 
		unsigned int var)
{
   unsigned int flags = 0;
   struct adslAturIntvlInfo adslAturIntvlStruct;
   memset(&adslAturIntvlStruct,0,sizeof(adslAturIntvlInfo));
   adslAturIntvlStruct.ifIndex= (*(int *)data_context);
   adslAturIntvlStruct.IntervalNumber= (*(int *)ret_len);
  
  switch (var) {

    case COLUMN_ADSLATURINTERVALLOFS:
  	flags = ATUR_INTVL_LOF_FLAG;
        break;
	
    case COLUMN_ADSLATURINTERVALLOSS:
	flags = ATUR_INTVL_LOS_FLAG;
	break;
	
    case COLUMN_ADSLATURINTERVALLPRS:
	flags = ATUR_INTVL_LPR_FLAG;
	break;
	
    case COLUMN_ADSLATURINTERVALESS:
	flags = ATUR_INTVL_ESS_FLAG;
	break;

    default:
     	DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
        return NULL;
  }

  SET_FLAG(&adslAturIntvlStruct.flags, flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_INTVL_INFO, &adslAturIntvlStruct);
#else
    get_adslAturIntervalTable(Fd, GET_ADSL_ATUR_INTVL_INFO,
        (void *) &adslAturIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAturIntvlStruct.flags, flags) == 0) 
       DEBUGMSGTL(("adslMIB-ATUR", "IntervalTable: GET- %x CMV fail\n", flags));

    switch (flags)  {

	 case ATUR_INTVL_LOF_FLAG:
    	    long_ret = adslAturIntvlStruct.intervalLOF;
            break;
	 
	 case ATUR_INTVL_LOS_FLAG:
    	    long_ret = adslAturIntvlStruct.intervalLOS;
	    break;
	  
	 case ATUR_INTVL_LPR_FLAG:
    	    long_ret = adslAturIntvlStruct.intervalLPR;
	    break;
	  
	 case ATUR_INTVL_ESS_FLAG:
    	    long_ret = adslAturIntvlStruct.intervalES;
	    break;
	  
         default:
     	    DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
            return NULL;
     }
     
    CLR_FLAG(&adslAturIntvlStruct.flags, flags);
    *ret_len = sizeof(u_int32);
    return (u_int32 *) &long_ret;
}

#else

/** XXX: return a data pointer to the data for the adslAturIntervalLofs column
    and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAturIntervalLofs(void *data_context, size_t * ret_len)
{
    struct adslAturIntvlInfo adslAturIntvlStruct;

    memset(&adslAturIntvlStruct,0,sizeof(adslAturIntvlInfo));
    adslAturIntvlStruct.ifIndex= (*(int *)data_context);
    adslAturIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_LOF_FLAG);
    printf("\n Flags = %0x \n", adslAturIntvlStruct.flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_INTVL_INFO, &adslAturIntvlStruct);
#else
    get_adslAturIntervalTable(Fd, GET_ADSL_ATUR_INTVL_INFO,
            (void *) &adslAturIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAturIntvlStruct.flags, ATUR_INTVL_LOF_FLAG) == 0) {
        printf ("\n The adsl_atur_intvl_Lofs could not return correct value\n");
    }
    else {
        printf ("\n The adsl_atur_intvl_Lofs value = %d\n",
            adslAturIntvlStruct.intervalLOF);
    }
    CLR_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_LOF_FLAG);
    long_ret = adslAturIntvlStruct.intervalLOF;
    *ret_len = sizeof(adslAturIntvlStruct.intervalLOF);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAturIntervalLoss column
    and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAturIntervalLoss(void *data_context, size_t * ret_len)
{
    struct adslAturIntvlInfo adslAturIntvlStruct;

    memset(&adslAturIntvlStruct,0,sizeof(adslAturIntvlInfo));
    adslAturIntvlStruct.ifIndex= (*(int *)data_context);
    adslAturIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_LOS_FLAG);
    printf("\n Flags = %0x \n", adslAturIntvlStruct.flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_INTVL_INFO, &adslAturIntvlStruct);
#else
    get_adslAturIntervalTable(Fd, GET_ADSL_ATUR_INTVL_INFO,
            (void *) &adslAturIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAturIntvlStruct.flags, ATUR_INTVL_LOS_FLAG) == 0) {
        printf ("\n The adsl_atur_intvl_Loss could not return correct value\n");
    }
    else {
        printf ("\n The adsl_atur_intvl_Loss value = %d\n",
            adslAturIntvlStruct.intervalLOS);
    }
    CLR_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_LOS_FLAG);
    long_ret = adslAturIntvlStruct.intervalLOS;
    *ret_len = sizeof(adslAturIntvlStruct.intervalLOS);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAturIntervalLprs column
    and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAturIntervalLprs(void *data_context, size_t * ret_len)
{
    struct adslAturIntvlInfo adslAturIntvlStruct;

    memset(&adslAturIntvlStruct,0,sizeof(adslAturIntvlInfo));
    adslAturIntvlStruct.ifIndex= (*(int *)data_context);
    adslAturIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_LPR_FLAG);
    printf("\n Flags = %0x \n", adslAturIntvlStruct.flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_INTVL_INFO, &adslAturIntvlStruct);
#else
    get_adslAturIntervalTable(Fd, GET_ADSL_ATUR_INTVL_INFO,
        (void *) &adslAturIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAturIntvlStruct.flags, ATUR_INTVL_LPR_FLAG) == 0) {
        printf ("\n The adsl_atur_intvl_Lprs could not return correct value\n");
    }
    else {
        printf ("\n The adsl_atur_intvl_Lprs value = %d\n",
            adslAturIntvlStruct.intervalLPR);
    }
    CLR_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_LPR_FLAG);
    long_ret = adslAturIntvlStruct.intervalLPR;
    *ret_len = sizeof(adslAturIntvlStruct.intervalLPR);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAturIntervalESs column
    and set ret_len to its proper size in bytes. */
u_int32      *
get_adslAturIntervalESs(void *data_context, size_t * ret_len)
{
    struct adslAturIntvlInfo adslAturIntvlStruct;

    memset(&adslAturIntvlStruct,0,sizeof(adslAturIntvlInfo));
    adslAturIntvlStruct.ifIndex= (*(int *)data_context);
    adslAturIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_ESS_FLAG);
    printf("\n Flags = %0x \n", adslAturIntvlStruct.flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_INTVL_INFO, &adslAturIntvlStruct);
#else
    get_adslAturIntervalTable(Fd, GET_ADSL_ATUR_INTVL_INFO,
        (void *) &adslAturIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAturIntvlStruct.flags, ATUR_INTVL_ESS_FLAG) == 0) {
        printf ("\n The adsl_atur_intvl_ESs could not return correct value\n");
    }
    else {
        printf ("\n The adsl_atur_intvl_ESs value = %d\n",
                adslAturIntvlStruct.intervalES);
    }
    CLR_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_ESS_FLAG);
    long_ret = adslAturIntvlStruct.intervalES;
    *ret_len = sizeof(adslAturIntvlStruct.intervalES);
    return (u_int32 *) &long_ret;
}

#endif

/** XXX: return a data pointer to the data for the adslAturIntervalValidData
    column and set ret_len to its proper size in bytes. */
int           *
get_adslAturIntervalValidData(void *data_context, size_t * ret_len)
{
    struct adslAturIntvlInfo adslAturIntvlStruct;

    memset(&adslAturIntvlStruct,0,sizeof(adslAturIntvlInfo));
    adslAturIntvlStruct.ifIndex= (*(int *)data_context);
    adslAturIntvlStruct.IntervalNumber= (*(int *)ret_len);
    SET_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_VALID_DATA_FLAG);
    printf("\n Flags = %0x \n", adslAturIntvlStruct.flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_INTVL_INFO, &adslAturIntvlStruct);
#else
    get_adslAturIntervalTable(Fd, GET_ADSL_ATUR_INTVL_INFO,
        (void *) &adslAturIntvlStruct);
#endif
    if (IS_FLAG_SET(&adslAturIntvlStruct.flags, ATUR_INTVL_VALID_DATA_FLAG)
            == 0) {
        printf ("\n The adsl_atur_intvl_valid_data could not return correct \
            value\n");
    }
    else {
        printf ("\n The adsl_atur_intvl_valid_data value = %d\n",
            adslAturIntvlStruct.intervalValidData);
    }
    CLR_FLAG(&adslAturIntvlStruct.flags, ATUR_INTVL_VALID_DATA_FLAG);
    long_ret = adslAturIntvlStruct.intervalValidData;
    *ret_len = sizeof(adslAturIntvlStruct.intervalValidData);
    return (int *) &long_ret;
}


#ifndef ENABLE_DRIVER_CALL
int get_adslAturIntervalTable(int Fd2,int GET_adsl_ATUR_INTVL_INFO2,
            void *adslAturInvtlTable)
{
    struct adslAturIntvlInfo *adslAturInvtlValue;
    adslAturInvtlValue = (adslAturIntvlInfo *)adslAturInvtlTable;
    printf ("\n File Descriptor fd = %d \n", Fd2);
    printf ("\n Ifindex  = %d \n", adslAturInvtlValue->ifIndex);
    printf ("\n GET_adsl_ATUR_INTVL_INFO2 = %d \n", GET_adsl_ATUR_INTVL_INFO2);

    adslAturInvtlValue->intervalLOF = 100;
    adslAturInvtlValue->intervalLOS = 200;
    adslAturInvtlValue->intervalLPR = 300;
    adslAturInvtlValue->intervalES = 400;
    adslAturInvtlValue->intervalValidData = 2;
    printf("\n Flags = %0x \n", adslAturInvtlValue->flags);
    return 0;
}

#endif

