/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    : adslLineTable_access.c
 * Author       : Subbi
 * Date         : 17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      : Amazon
 * Block        : ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                 1) Getting the data context pointer & loop context pointer
 *                 2) User-defined data access functions (per column) for data
 *                    in table adslLineTable
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */

/* ===========================================================================
 * Revision History:
 *
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.access_functions.conf,v 1.6 2003/09/11 22:48:49 rstory Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
 */
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslLineTable_access.h"
#include "adslLineTable_enums.h"

/*
 * ===========================================================================
 *                           GLOBAL DEFINITIONS
 * ===========================================================================
 */
// *** Global variable definitions
static long long_ret;

/*
 * ===========================================================================
 *                           LOCAL FUNCTIONS
 * ===========================================================================
 */

/** returns the first data point within the adslLineTable table data.

    Set the my_loop_context variable to the first data point structure
    of your choice (from which you can find the next one).  This could
    be anything from the first node in a linked list, to an integer
    pointer containing the beginning of an array variable.

    Set the my_data_context variable to something to be returned to
    you later that will provide you with the data to return in a given
    row.  This could be the same pointer as what my_loop_context is
    set to, or something different.

    The put_index_data variable contains a list of snmp variable
    bindings, one for each index in your table.  Set the values of
    each appropriately according to the data matching the first row
    and return the put_index_data variable at the end of the function.
*/
netsnmp_variable_list *
adslLineTable_get_first_data_point(void **my_loop_context,
                                   void **my_data_context,
                                   netsnmp_variable_list * put_index_data,
                                   netsnmp_iterator_info *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-First: adslLineTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }
    *My_Data_Context=IFTYPE_ADSL;
    *my_data_context=My_Data_Context;

    put_index_data = adslTable_get_first_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;

}

/** functionally the same as adslLineTable_get_first_data_point, but
   my_loop_context has already been set to a previous value and should
   be updated to the next in the list.  For example, if it was a
   linked list, you might want to cast it to your local data type and
   then return my_loop_context->next.  The my_data_context pointer
   should be set to something you need later and the indexes in
   put_index_data updated again. */
netsnmp_variable_list *
adslLineTable_get_next_data_point(void **my_loop_context,
                                  void **my_data_context,
                                  netsnmp_variable_list * put_index_data,
                                  netsnmp_iterator_info *mydata)
{

/* Go the data point where the my_loop_context pointer points to
   Search for the next adsl entry in the IfTable
   Update the put_index_data,loop_context and data_context pointers
*/

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
        put_index_data = NULL;
        snmp_log(LOG_ERR, "Get-Next: adslLineTable: My_Data_Context malloc failed\n");
        return put_index_data;
    }
    *My_Data_Context=IFTYPE_ADSL;
    *my_data_context=My_Data_Context;

     put_index_data = adslTable_get_next_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}


/*  User-defined data access functions (per column) for data in table
    adslLineTable */
/*
 * NOTE:
 * - these get_ routines MUST return data that will not be freed (ie,
 *   use static variables or persistent data).  It will be copied, if
 *   needed, immediately after the get_ routine has been called.
 * - these SET routines must copy the incoming data and can not take
 *   ownership of the memory passed in by the val pointer.
 */
/** XXX: return a data pointer to the data for the adslLineCoding column and set
         ret_len to its proper size in bytes. */
int           *
get_adslLineCoding(void *data_context, size_t * ret_len)
{

    struct adslLineTableEntry adslLineTableStruct;
    memset(&adslLineTableStruct,0,sizeof(adslLineTableEntry));
    adslLineTableStruct.ifIndex=*((int *)data_context);
    SET_FLAG(&adslLineTableStruct.flags,LINE_CODE_FLAG);

#ifdef ENABLE_DRIVER_CALL
     adsl_ioctl(Fd, GET_ADSL_LINE_CODE, (void *)&adslLineTableStruct);
#else
     get_adslLineTable(Fd, GET_ADSL_LINE_CODE,(void *)&adslLineTableStruct);
#endif
    if (IS_FLAG_SET(&adslLineTableStruct.flags, LINE_CODE_FLAG) == 0)
        DEBUGMSGTL(("adslMIB-Line", "LineCode CMV fail\n"));

    CLR_FLAG(&adslLineTableStruct.flags, LINE_CODE_FLAG);
    long_ret = adslLineTableStruct.adslLineCode;
    *ret_len = sizeof(adslLineTableStruct.adslLineCode);
    return (int *) &long_ret;
}


#ifndef ENABLE_DRIVER_CALL
int get_adslLineTable(int Fd2, int Adsl_Line_CODE,void *adslLineTable)
{
   struct adslLineTableEntry *adslLineTableValue;
   adslLineTableValue = (adslLineTableEntry *)adslLineTable;
   adslLineTableValue->adslLineCode = 2;
   return 0;
 }
#endif

